@@ -126,6 +126,14 @@ IRECV_DEVICES = (
                 display_name='iPhone 16 Pro'),
     IRecvDevice(product_type='iPhone17,2', hardware_model='d94ap', board_id=0x0E, chip_id=0x8140,
                 display_name='iPhone 16 Pro Max'),
+    IRecvDevice(product_type='iPhone18,4', hardware_model='d23ap', board_id=0x0A, chip_id=0x8150,
+                display_name='iPhone Air'),
+    IRecvDevice(product_type='iPhone18,1', hardware_model='v53ap', board_id=0x0C, chip_id=0x8150,
+                display_name='iPhone 17 Pro'),
+    IRecvDevice(product_type='iPhone18,2', hardware_model='v54ap', board_id=0x0E, chip_id=0x8150,
+                display_name='iPhone 17 Pro Max'),
+    IRecvDevice(product_type='iPhone18,3', hardware_model='v57ap', board_id=0x08, chip_id=0x8150,
+                display_name='iPhone 17'),
     # iPod
     IRecvDevice(product_type='iPod1,1', hardware_model='n45ap', board_id=0x02, chip_id=0x8900,
                 display_name='iPod Touch (1st gen)'),
@@ -412,6 +420,24 @@ IRECV_DEVICES = (
                 display_name='Apple Watch Series 10 (42mm Cellular)'),
     IRecvDevice(product_type='Watch7,11', hardware_model='n218bap', board_id=0x16, chip_id=0x8310,
                 display_name='Apple Watch Series 10 (46mm Cellular)'),
+    IRecvDevice(product_type='Watch7,12', hardware_model='n230ap', board_id=0x22, chip_id=0x8310,
+                display_name='Apple Watch Ultra 3'),
+    IRecvDevice(product_type='Watch7,13', hardware_model='n243sap', board_id=0x28, chip_id=0x8310,
+                display_name='Apple Watch SE 3 (40mm)'),
+    IRecvDevice(product_type='Watch7,14', hardware_model='n243bap', board_id=0x2A, chip_id=0x8310,
+                display_name='Apple Watch SE 3 (44mm)'),
+    IRecvDevice(product_type='Watch7,15', hardware_model='n244sap', board_id=0x2C, chip_id=0x8310,
+                display_name='Apple Watch SE 3 (40mm Cellular)'),
+    IRecvDevice(product_type='Watch7,16', hardware_model='n244bap', board_id=0x2E, chip_id=0x8310,
+                display_name='Apple Watch SE 3 (44mm Cellular)'),
+    IRecvDevice(product_type='Watch7,17', hardware_model='n227sap', board_id=0x18, chip_id=0x8310,
+                display_name='Apple Watch Series 11 (42mm)'),
+    IRecvDevice(product_type='Watch7,18', hardware_model='n227bap', board_id=0x1A, chip_id=0x8310,
+                display_name='Apple Watch Series 11 (46mm)'),
+    IRecvDevice(product_type='Watch7,19', hardware_model='n228sap', board_id=0x1C, chip_id=0x8310,
+                display_name='Apple Watch Series 11 (42mm Cellular)'),
+    IRecvDevice(product_type='Watch7,20', hardware_model='n228bap', board_id=0x1E, chip_id=0x8310,
+                display_name='Apple Watch Series 11 (46mm Cellular)'),
     # Apple Silicon Macs
     IRecvDevice(product_type='ADP3,2', hardware_model='j273aap', board_id=0x42, chip_id=0x8027,
                 display_name='Developer Transition Kit (2020)'),
