@@ -1,4 +1,4 @@
-from pymobiledevice3.exceptions import PyMobileDevice3Exception
+from pymobiledevice3.exceptions import AppNotInstalledError, PyMobileDevice3Exception
 from pymobiledevice3.lockdown import LockdownClient
 from pymobiledevice3.lockdown_service_provider import LockdownServiceProvider
 from pymobiledevice3.services.afc import AfcService, AfcShell
@@ -23,13 +23,20 @@ class HouseArrestService(AfcService):
         else:
             cmd = VEND_CONTAINER
         self.documents_only = documents_only
-        self.send_command(bundle_id, cmd)
+        try:
+            self.send_command(bundle_id, cmd)
+        except PyMobileDevice3Exception:
+            self.close()
+            raise
 
     def send_command(self, bundle_id: str, cmd: str = 'VendContainer') -> None:
         response = self.service.send_recv_plist({'Command': cmd, 'Identifier': bundle_id})
         error = response.get('Error')
         if error:
-            raise PyMobileDevice3Exception(error)
+            if error == 'ApplicationLookupFailed':
+                raise AppNotInstalledError(f'No app with bundle id {bundle_id} found')
+            else:
+                raise PyMobileDevice3Exception(error)
 
     def shell(self) -> None:
         AfcShell.create(self.lockdown, service=self, auto_cd=DOCUMENTS_ROOT if self.documents_only else '/')
